/*
  LiquidCrystal Library - Hello World
 
 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the 
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.
 
 Ce programme gère l'affichage de la bobineuse et la gestion moteur
 Affiche sur l'écran LCD le nombre de spires
 Gère l'avance du moteur pàp en fonction du diamètre
 
  Le circuit:
 * ILS 1 to digital pin 3
 * ILS 2 to digital pin 2
 * LCD RS pin to digital pin 7
 * LCD Enable pin to digital pin 11
 * LCD D4 pin to digital pin 6
 * LCD D5 pin to digital pin 8
 * LCD D6 pin to digital pin 9
 * LCD D7 pin to digital pin 10
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)
 
 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe
 
 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystal
 */

// include the library code:
#include <LiquidCrystal.h>
#include <Stepper.h>
// Définition des PIN
#define ILS_A  3     
#define ILS_B  2  

// Définition des variables
// Mémorise l'état des sorties A et B
int StateA = 0; 
int StateB = 0;
// Mémorise l'état des sorties A et B antérieures
int StateA_anterieur = 0; 
int StateB_anterieur = 0; 

int Compteur = 0;
int Compteur_anterieur = 0;
int Compteur_impulsion = 0;
int A = 0;
int B = 0;
int C = 0;
int D = 0;


float Nbspire = 0;
int Sense = 0;
int Steps = 0;

int val = 0;
int stepsMaped = 0;                   // for your motor
// Mémorise le diamètre du fil
float fFil = 0;
// Define various ADC prescaler
const unsigned char PS_16 = (1 << ADPS2);
const unsigned char PS_128 = (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(7, 11, 10, 9, 8, 6);

// initialize the stepper library on pins 8 through 11:
// 48 is the number of steps per revolution,change this to fit the number of steps per revolution
// Réglage pour le moteur 55SI-25DAWC
Stepper myStepper(48,4,12,5,13);

void setup() {
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print("Compteur");
  // Initialise les entrées A et B de l'encodeur
  pinMode(ILS_A, INPUT); 
  pinMode(ILS_B, INPUT);
  // set up the ADC
  ADCSRA &= ~PS_128;  // remove bits set by Arduino library
  // you can choose a prescaler from above.
  // PS_16, PS_32, PS_64 or PS_128
  ADCSRA |= PS_16;    
  // set our own prescaler to 16 
  // set the speed at 300 rpm:
  myStepper.setSpeed(300); 
}

void loop() {
  
    Comptage();
    
    Compteimpulsion();
    
    AfficheCompteur();
    
    Diamfil();
    
    AfficheDiamFil();
    
    if ( A <= 7 )
    {
      B = int(stepsMaped*0.01);
    }
    else if ( A == 8 )
    {
      C = int((stepsMaped*0.1)-(int(stepsMaped*0.01)*8));
      B = (C*0.5);
    }
    else if ( A == 9 )
    {
      C = int(int((stepsMaped*0.1)-(int(stepsMaped*0.01)*8))*0.5);
      D = (int(stepsMaped*0.01))*8;
      B = ((int(stepsMaped*0.1))-D)-C;            
    }
        
    Avanceguidefil();
     
 
}


// Fonction de comptage
void Comptage()
{
    // Lecture des sorties A et B de l'encodeur. Avant chaque lecture,
  // mémorisation de l'état antérieur de chaque broche. Ce principe permet
  // la détection d'un front montant.
  StateA_anterieur= StateA;
  StateA = digitalRead(ILS_A);
  StateB_anterieur= StateB;  
  StateB = digitalRead(ILS_B);

  // Détection du front montant :
  // Etat actuel : haut, état antérieur : bas
  if ((StateA == HIGH) && (StateA_anterieur == LOW)) { 
    if (StateB == LOW) {
      Compteur = Compteur++;
      Sense = 1;      
      }
    
    };  

  // Détection du front montant :
  // Etat actuel : haut, état antérieur : bas
  if ((StateB == HIGH) && (StateB_anterieur == LOW)) { 
    if (StateA == LOW) {
      Compteur = Compteur--;
      Sense = -1;      
      };    
    }
}

// Fonction comptage des impulsions
void Compteimpulsion()
{
  A = abs(Compteur_impulsion);
  
  if (A<10)
  {
   Compteur_impulsion = Compteur_impulsion + Sense;
  }
  else 
  {
   Compteur_impulsion = 0;
  } 

}

// Fonction d'affichage du compteur
void AfficheCompteur()
{
  Nbspire=Compteur*0.1;
  // set the cursor to column 12, line 0
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(11,0);
  // reset the screen:
  lcd.print("    ");
  // set the cursor to column 12, line 0
  lcd.setCursor(11,0);
  //lcd.print(Nbspire,1); 
  lcd.print(Compteur_impulsion);
}

// Fonction de paramétrage du diamètre du fil
void Diamfil()
{
  val=analogRead(A5);
  stepsMaped=map(val, 0, 1023, 0, 960);
      
  // Doit etre calculé par rapport à la tige filetée 
  // 1 tour de tige filetée en pas de 1 revient à une avance de 1 mm
  // Soit 1/48 (par rapport au moteur)steps fois le nombre de steps d'avance
  // pour faire 1 tour complet du motoréducteur 
  fFil = stepsMaped*0.002083333;
}
// Fonction d'affichage du diamètre du fil
void AfficheDiamFil()
{
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0,1);
  // print the number of seconds since reset:
  lcd.print("Diam fil");
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(11,1);
  lcd.print(fFil,3); 
}

// Fonction D'avance du guide fil 
void Avanceguidefil()
{
  // revolution  in one direction or the other:
  Steps = (Sense*B);
  myStepper.step(Steps);
  Sense = 0;
  B = 0;
  C = 0;
  D = 0;
}
